@@ -16,7 +16,7 @@ from ...core import geodetic
 
 def test_system_wgs84():
     """Checking expected WGS-84 properties."""
-    wgs84 = core.geodetic.System()
+    wgs84 = core.geodetic.Spheroid()
     # https://fr.wikipedia.org/wiki/WGS_84
     # https://en.wikipedia.org/wiki/Geodetic_datum
     # http://earth-info.nga.mil/GandG/publications/tr8350.2/wgs84fin.pdf
@@ -48,9 +48,9 @@ def test_system_wgs84():
 
 def test_system_operators():
     """Test operators."""
-    wgs84 = core.geodetic.System()
+    wgs84 = core.geodetic.Spheroid()
     # https://en.wikipedia.org/wiki/Geodetic_Reference_System_1980
-    grs80 = core.geodetic.System(6378137, 1 / 298.257222101)
+    grs80 = core.geodetic.Spheroid(6378137, 1 / 298.257222101)
     assert 6378137 == pytest.approx(grs80.semi_major_axis)
     assert 1 / 298.257222101 == pytest.approx(grs80.flattening)
     assert wgs84 == wgs84
@@ -59,7 +59,7 @@ def test_system_operators():
 
 def test_system_pickle():
     """Serialization test."""
-    wgs84 = core.geodetic.System()
+    wgs84 = core.geodetic.Spheroid()
     assert wgs84 == pickle.loads(pickle.dumps(wgs84))
 
 
@@ -359,6 +359,15 @@ def test_crossover():
     assert list(crossover.half_orbit_1) == list(other.half_orbit_1)
     assert list(crossover.half_orbit_2) == list(other.half_orbit_2)
 
+    for flag in (True, False):
+        crossover_properties = core.geodetic.calculate_crossover(
+            lon1, lat1, lon2, lat2, cartesian_plane=flag)
+        assert crossover_properties is not None
+        point, (ix1, ix2) = crossover_properties
+        assert (ix1, ix2) == (3, 3) if flag else (4, 3)
+        assert pytest.approx(point.lon) == 4
+        assert pytest.approx(point.lat, rel=1e-3) == 4.0018282189756835
+
 
 def test_merged_point():
     """Try to compute a crossover from overlay tracks."""
@@ -379,6 +388,16 @@ def test_merged_point():
     assert not crossover.exists()
     assert crossover.search() is None
 
+    for flag in (True, False):
+        with pytest.raises(RuntimeError):
+            core.geodetic.calculate_crossover(lon1,
+                                              lat1,
+                                              lon1,
+                                              lat1,
+                                              cartesian_plane=flag)
+        crossover_properties = core.geodetic.calculate_crossover(
+            lon1, lat1, lon1 + 10, lat1 + 10, cartesian_plane=flag)
+
 
 def test_missing_crossover():
     """Try to calculate a crossing point when the entry passes do not cross."""
@@ -392,6 +411,11 @@ def test_missing_crossover():
     assert isinstance(crossover, core.geodetic.Crossover)
     assert crossover.exists()
 
+    for flag in (True, False):
+        crossover_properties = core.geodetic.calculate_crossover(
+            x1, y1, x2, y2, cartesian_plane=flag)
+        assert crossover_properties is not None
+
 
 def test_case_crossover_shift():
     """Calculate a crossover between two tracks."""
@@ -410,6 +434,16 @@ def test_case_crossover_shift():
     assert pytest.approx(coordinates.lat, rel=1e-3) == 1.6551107341906504
     assert crossover.nearest(coordinates, predicate=None) == (2, 1)
 
+    for flag in (True, False):
+        crossover_properties = core.geodetic.calculate_crossover(
+            lon1, lat1, lon2, lat2, cartesian_plane=flag)
+        assert crossover_properties is not None
+        point, (ix1, ix2) = crossover_properties
+        assert (ix1, ix2) == (1, 1) if flag else (2, 1)
+        assert pytest.approx(point.lon) == 15
+        assert pytest.approx(point.lat,
+                             rel=1e-3) == (1.5 if flag else 1.6551107341906504)
+
 
 def test_bbox_from_geojson():
     """Calculate a bounding box from a geojson coordinate array."""
@@ -558,3 +592,43 @@ def test_linestring():
     b.append(core.geodetic.Point(2, 3))
     b.append(core.geodetic.Point(3, 4))
     assert a == b
+
+
+def test_curvilinear_distance():
+    """Test the curvilinear distance calculation."""
+    lon = np.array([
+        -9.72270435, -9.58849868, -9.45429341, -9.32009001, -9.18588996,
+        -9.05169472, -8.91750577, -8.78332458, -8.64915263, -8.51499137,
+        -8.38084228, -8.24670684, -8.1125865, -7.97848274, -7.84439702,
+        -7.71033081, -7.57628556, -7.44226275, -7.30826383, -7.17429026,
+        -7.04034349, -6.906425, -6.77253622, -6.63867861, -6.50485362,
+        -6.37106269, -6.23730728, -6.10358883, -5.96990877, -5.83626855,
+        -5.70266959, -5.56911334, -5.43560123, -5.30213467, -5.1687151,
+        -5.03534394, -4.9020226, -4.7687525, -4.63553505, -4.50237166,
+        -4.36926374, -4.23621268, -4.10321989, -3.97028675, -3.83741466,
+        -3.70460501, -3.57185917, -3.43917852, -3.30656444, -3.1740183
+    ])
+    lat = np.array([
+        -77.66299965, -77.66292905, -77.66279284, -77.66259101, -77.66232357,
+        -77.66199051, -77.66159184, -77.66112757, -77.66059769, -77.66000222,
+        -77.65934117, -77.65861453, -77.65782234, -77.65696458, -77.65604129,
+        -77.65505246, -77.65399812, -77.65287828, -77.65169296, -77.65044218,
+        -77.64912596, -77.64774431, -77.64629727, -77.64478485, -77.64320707,
+        -77.64156397, -77.63985556, -77.63808189, -77.63624296, -77.63433883,
+        -77.63236951, -77.63033504, -77.62823545, -77.62607078, -77.62384106,
+        -77.62154633, -77.61918663, -77.616762, -77.61427247, -77.61171809,
+        -77.6090989, -77.60641495, -77.60366627, -77.60085292, -77.59797493,
+        -77.59503237, -77.59202528, -77.5889537, -77.5858177, -77.58261731
+    ])
+    ls = geodetic.LineString(lon, lat)
+
+    distance = [0.0]
+    for ix in range(1, 50):
+        distance.append(
+            geodetic.Point(lon[ix - 1],
+                           lat[ix - 1]).distance(geodetic.Point(
+                               lon[ix], lat[ix]),
+                                                 strategy="thomas"))
+
+    assert np.all((ls.curvilinear_distance(strategy="thomas") -
+                   np.cumsum(np.array(distance))) == np.zeros(50))
